"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EError = void 0;
const EMPTY_STACK_TRACE = "<stack trace empty>";
class EError extends Error {
    constructor(messageOrCause = "", optionsOrCause) {
        var _a, _b;
        let _message = "";
        let _cause = undefined;
        let _info = undefined;
        if (messageOrCause instanceof Error) {
            _message = "";
            _cause = messageOrCause;
        }
        else if (optionsOrCause instanceof Error) {
            _message = messageOrCause;
            _cause = optionsOrCause;
        }
        else if (optionsOrCause) {
            _message = messageOrCause;
            const { cause, info } = optionsOrCause;
            _cause = cause;
            _info = info;
        }
        else {
            _message = messageOrCause;
        }
        if (!_cause) {
            super(_message);
        }
        else {
            const causes = EError.getCauses(_cause);
            const messages = [];
            if (_message)
                messages.push(_message);
            causes.forEach((cause) => {
                const messageString = [cause.name];
                if (cause instanceof EError) {
                    if (cause.originalMessage)
                        messageString.push(cause.originalMessage);
                }
                else {
                    if (cause.message)
                        messageString.push(cause.message);
                }
                messages.push(messageString.join(": "));
            });
            super(messages.join(" > "));
        }
        this.originalMessage = _message;
        if (_cause)
            this.cause = _cause;
        if (_info)
            this.info = _info;
        Object.setPrototypeOf(this, new.target.prototype);
        this.name = new.target.name;
        (_b = (_a = Error).captureStackTrace) === null || _b === void 0 ? void 0 : _b.call(_a, this, new.target);
    }
    static getCauses(error, filter) {
        const causes = [];
        let currentError = error;
        while (currentError) {
            if (!filter || filter(currentError))
                causes.push(currentError);
            currentError = currentError.cause;
        }
        return causes;
    }
    getCauses(filter) {
        return EError.getCauses(this, filter);
    }
    static findCause(error, type) {
        let errorCause = error;
        while (errorCause) {
            if (errorCause instanceof type && errorCause.name === type.name)
                return errorCause;
            errorCause = errorCause.cause;
        }
        return null;
    }
    findCause(type) {
        return EError.findCause(this, type);
    }
    static findCauses(error, type) {
        return EError.getCauses(error, (e) => e instanceof type && e.name === type.name);
    }
    findCauses(type) {
        return EError.getCauses(this, (e) => e instanceof type && e.name === type.name);
    }
    static findCauseByName(error, name) {
        let errorCause = error;
        while (errorCause) {
            if (errorCause instanceof Error && errorCause.name === name)
                return errorCause;
            errorCause = errorCause.cause;
        }
        return null;
    }
    findCauseByName(name) {
        return EError.findCauseByName(this, name);
    }
    static findCausesByName(error, name) {
        return EError.getCauses(error, (e) => e.name === name);
    }
    findCausesByName(name) {
        return EError.findCausesByName(this, name);
    }
    static fullStack(error) {
        var _a;
        let stack = (_a = error.stack) !== null && _a !== void 0 ? _a : EMPTY_STACK_TRACE;
        if (error.cause)
            stack += "\n\n" + `caused by: ${EError.fullStack(error.cause)}`;
        return stack;
    }
    fullStack() {
        return EError.fullStack(this);
    }
    static toJSON(error, options = {}) {
        const { stack, shallow } = options;
        const json = {
            name: error.name,
            message: error.message,
        };
        if (error instanceof EError && typeof error.originalMessage === "string") {
            json.originalMessage = error.originalMessage;
        }
        if (!shallow && error.cause) {
            json.cause = EError.toJSON(error.cause, options);
        }
        if (error instanceof EError && error.info) {
            json.info = error.info;
        }
        if (stack) {
            const stack = error.stack;
            if (stack)
                json.stack = stack.split("\n");
        }
        return json;
    }
    toJSON(options = {}) {
        return EError.toJSON(this, options);
    }
}
exports.EError = EError;
